/*
 * Copyright © 2013 Sterela
 * Copyright © 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/** @file */

#ifndef _IVMOB_CAN_H
#define _IVMOB_CAN_H


/** @name Introduction
 * @brief CAN Bus interface definitions
 *
 * From Sterela User Manual
 */

/** @name CAN Commands ids */
/**@{*/
#define IVMOB_SPEED_CMD			0x010
#define IVMOB_ACTIVE_CMD			0x012
#define IVMOB_INACTIVE_CMD		0x014
#define IVMOB_LASER_SCANNER_CMD		0x016
#define IVMOB_PROCESSED_SPEED_RQST	0x040
#define IVMOB_FRONT_WHEEL_SPEED_RQST	0x042
#define IVMOB_REAR_WHEEL_SPEED_RQST	0x044
#define IVMOB_FRONT_CURRENT_RQST		0x046
#define IVMOB_REAR_CURRENT_RQST		0x048
#define IVMOB_STATUS_RQST		0x04a
#define IVMOB_SOFTWARE_VERSION_RQST	0x04c
#define IVMOB_COMMISSIONING_DATE_RQST	0x04e
#define IVMOB_TOTAL_DISTANCE_RQST	0x050
#define IVMOB_TOTAL_ON_TIME_RQST		0x052
#define IVMOB_WHEEL_SIZE_RQST		0x054
#define IVMOB_TORQUE_CONSTANT_RQST	0x056
#define IVMOB_GEAR_RATIO_RQST		0x058
#define IVMOB_WHEEL_TRACK_WIDTH_RQST	0x05a
#define IVMOB_LASER_SCANNER_RQST		0x05C
#define IVMOB_PERIODIC_MSG_CONFIG	0x070
#define IVMOB_PAYLOAD_OFF_CONFIRM	0x0A5
/**@}*/

/** @name CAN Responses ids */
/**@{*/
#define IVMOB_RESP_SPEED_CMD		0x011
#define IVMOB_RESP_ACTIVE_CMD		0x013
#define IVMOB_RESP_INACTIVE_CMD		0x015
#define IVMOB_RESP_LASER_SCANNER_CMD	0x017
#define IVMOB_RESP_PROCESSED_SPEED	0x041
#define IVMOB_RESP_FRONT_WHEEL_SPEED	0x043
#define IVMOB_RESP_REAR_WHEEL_SPEED	0x045
#define IVMOB_RESP_FRONT_CURRENT		0x047
#define IVMOB_RESP_REAR_CURRENT		0x049
#define IVMOB_RESP_STATUS		0x04b
#define IVMOB_RESP_SOFTWARE_VERSION	0x04d
#define IVMOB_RESP_COMMISSIONING_DATE	0x04f
#define IVMOB_RESP_TOTAL_DISTANCE	0X051
#define IVMOB_RESP_TOTAL_ON_TIME		0x053
#define IVMOB_RESP_WHEEL_SIZE		0x055
#define IVMOB_RESP_TORQUE_CONSTANT	0x057
#define IVMOB_RESP_GEAR_RATIO		0x059
#define IVMOB_RESP_WHEEL_TRACK_WIDTH	0x05b
#define IVMOB_RESP_LASER_SCANNER		0x05d
#define IVMOB_RESP_PERIODIC_MSG		0x071
/**@}*/

/** @name Periodic CAN data transfers */
/**@{*/
#define IVMOB_PROCESSED_SPEED  		0x081
#define IVMOB_FRONT_WHEEL_SPEED		0x083
#define IVMOB_REAR_WHEEL_SPEED		0x085
#define IVMOB_FRONT_CURRENT		0x087
#define IVMOB_REAR_CURRENT		0x089
#define IVMOB_STATUS			0x08b
/**@}*/

/** @name Spontaneous messages from the robot */
/**@{*/
#define IVMOB_ALARM_SAFETY_MODE		0x0a2
#define IVMOB_PAYLOAD_OFF_RQST		0x0a4
#define IVMOB_LOW_BATTERY		0x0a6
#define IVMOB_INVALID_COMMAND		0x0ff
/**@}*/

/**
 * @brief private communication data structure
 */
struct ivmob_private {
	int socket;		/**< CAN socket file descriptor */
	pthread_t tid;		/**< thread id of the read task  */
	pthread_mutex_t mutex;	/**< lock for the ivmob structure */
	pthread_cond_t cond;	/**< condition variable for waiting answers */
	int16_t pending;	/**< pending answer message ID */
	int done;		/**< flag to end the read task  */
};

extern void *ivmobReadTask(void *);


#endif
