/*
 * Copyright (c) 2014 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <stdio.h>

#include "ivmob/ivmob.h"

/**
 * @brief code to name correspondance tables
 */
struct ivmob_name_table {
	int code;
	char *name;
};

static const char *ivmob_operating_modes[] = {
	"unknown(0)",
	"inactive",
	"active",
	"safety",
	"wait reset"
};

/**
 * @brief Returns the name of the given operating mode
 */
const char *
ivmob_operating_mode(int mode)
{
	static char buf[80];
	if (mode < 0 || mode > 4) {
		snprintf(buf, sizeof(buf), "unknown (%d)", mode);
		return buf;
	}
	return ivmob_operating_modes[mode];
}

static struct ivmob_name_table ivmob_safety_codes[] = {
	{ IVMOB_ERROR_FRONT_ESTOP, "front estop" },
	{ IVMOB_ERROR_REAR_ESTOP, "rear etop" },
	{ IVMOB_ERROR_RADIO_ESTOP, "radio estop" },
	{ IVMOB_ERROR_PAYLOAD_ESTOP, "payload estop" },
	{ IVMOB_ERROR_FRONT_BUMPER, "front bumber" },
	{ IVMOB_ERROR_REAR_BUMPER, "rear bumper" },
	{ IVMOB_ERROR_FRONT_LASER_SCANNER, "front laser scanner" },
	{ IVMOB_ERROR_REAR_LASER_SCANNER, "rear laser scanner" },
	{ IVMOB_ERROR_MOTOR_PROBLEM, "motor problem" },
	{ IVMOB_ERROR_INTERNAL_COMM, "internal communication error" },
	{ IVMOB_ERROR_PAYLOAD_COMM, "payload communication error" },
	{ IVMOB_ERROR_INTERNAL_ERROR, "internal error" },
	{ IVMOB_ERROR_BATTERY_SAFETY, "battery safety mode" },
	{ IVMOB_ERROR_CHARGER_CONNECTED, "charger connected" },
	{ IVMOB_ERROR_MOTION_DISABLED, "motion disabled" },
	{ IVMOB_ERROR_BREAKS_DISABLED, "breaks disabled" },
	{ IVMOB_ERROR_MODE_CHANGED, "mode changed" },
	{ IVMOB_ERROR_INITIALIZING, "initializing" },
	{ IVMOB_ERROR_OK, "ready" }
};

#define N_ELEMENTS(x) (sizeof(x)/sizeof(x[0]))

/**
 * @brief Returns the name of the given safety code
 */
const char *
ivmob_safety_code(int code)
{
	int i;

	for (i = 0; i < N_ELEMENTS(ivmob_safety_codes); i++)
		if (code == ivmob_safety_codes[i].code)
			return ivmob_safety_codes[i].name;
	return "unknow error code";
}
