/*
 * Copyright (c) 2013,2014 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <ivmob/ivmob.h>

int
main(int argc, char *argv[])
{
	struct ivmob *ivmob;
	int rc;
	char *device;

	if (argc == 1) 
		device = "can0";
	else if (argc == 2) 
		device = argv[1];
	else {
		fprintf(stderr, "usage: %s device\n", argv[0]);
		exit(2);
	}
	ivmob = ivmobInit(device, true);
	if (ivmob == NULL) {
		fprintf(stderr, "cannot init %s\n", argv[1]);
		exit(2);
	}
	if ((rc = ivmobRqstSoftwareVersion(ivmob, 2000)) != 0) {
		warn("ivmobRqstSoftwareVersion");
		ivmobEnd(ivmob);
		exit(2);
	}
	ivmobRqstStatus(ivmob, -1);
	ivmobRqstCommissioningDate(ivmob, -1);
	ivmobRqstTotalDistance(ivmob, -1);
	ivmobRqstTotalOnTime(ivmob, -1);
	printf("Software version: %d.%d.%d build %d\n", ivmob->version_major,
	    ivmob->version_minor, ivmob->version_tiny, ivmob->version_build);
	printf("Date            : %d/%02d/%02d\n", 
	    ivmob->commissioning_year+2000,
	    ivmob->commissioning_month,
	    ivmob->commissioning_day);
	printf("Total distance  : %.03lf km\n", ivmob->total_distance/1000.0);
	printf("ON time         : %d h %d mn\n", ivmob->total_on_time/60,
	    ivmob->total_on_time % 60);
	printf("Temperature     : %d C\n"
	       "Battery         : %d/8\n"
               "Mode            : %s\n"
               "Safety code     : %s\n",
	    ivmob->temperature, ivmob->battery_level, 
	    ivmob_operating_mode(ivmob->operational_mode), 
	    ivmob_safety_code(ivmob->safety_error_code));
	printf("Motion          : %s\n",
	    (ivmob->mmi & 0x4) ? "forbidden" : "authorized");
	printf("Charger         : %s\n", 
	    (ivmob->mmi & 0x02) ? "connected" : "disconnected");
	printf("Control selector: %s\n",
	    (ivmob->mmi & 0x01) ? "remote" : "local");
	ivmobRqstPeriodicMessages(ivmob, 0, 0, 0, 0, 0, 0, 0);
	ivmobEnd(ivmob);
	return 0;
}
