/*
 * Copyright (c) 2013,2014 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <ivmob/ivmob.h>

int
main(int argc, char *argv[])
{
	struct ivmob *ivmob;
	int i, rc, pending_active;
	char *device;

	if (argc == 1) 
		device = "can0";
	else if (argc == 2) 
		device = argv[1];
	else {
		fprintf(stderr, "usage: %s device\n", argv[0]);
		exit(2);
	}
	ivmob = ivmobInit(device, true);
	if (ivmob == NULL) {
		fprintf(stderr, "cannot init %s\n", argv[1]);
		exit(2);
	}
	if ((rc = ivmobRqstSoftwareVersion(ivmob, 2000)) != 0) {
		warn("ivmobRqstSoftwareVersion");
		ivmobEnd(ivmob);
		exit(2);
	}
	printf("requesting periodic updates\n");
	ivmobRqstPeriodicMessages(ivmob, 10, 1, 1, 1, 1, 1, 2000);
	pending_active = 0;
	while (1) {
		printf("Temperature     : %d C\n"
		    "Battery         : %d/8\n"
		    "Mode            : %s\n"
		    "Safety code     : %s\n",
		    ivmob->temperature, ivmob->battery_level, 
		    ivmob_operating_mode(ivmob->operational_mode), 
		    ivmob_safety_code(ivmob->safety_error_code));
		printf("Motion          : %s\n",
		    (ivmob->mmi & 0x4) ? "forbidden" : "authorized");
		printf("Charger         : %s\n", 
		    (ivmob->mmi & 0x02) ? "connected" : "disconnected");
		printf("Control selector: %s\n",
		    (ivmob->mmi & 0x01) ? "remote" : "local");
		printf("\n");
		if ((ivmob->mmi & 0x01) != 0) 
			printf("*** switch to local mode !!!\n");
		if (ivmob->operational_mode == IVMOB_MODE_INACTIVE &&
		    !pending_active) {
			ivmobCmdActive(ivmob, 2000);
			pending_active = 1;
		}
		if (ivmob->operational_mode == IVMOB_MODE_ACTIVE) 
			break;
		usleep(200000);
	}
	for (i = 0; i < 500; i++) {
		ivmobCmdSpeed(ivmob, 0, 0, 200);
		printf("Temperature     : %d C\n"
		    "Battery         : %d/8\n"
		    "Mode            : %s\n"
		    "Safety code     : %s\n",
		    ivmob->temperature, ivmob->battery_level, 
		    ivmob_operating_mode(ivmob->operational_mode), 
		    ivmob_safety_code(ivmob->safety_error_code));
		printf("Motion          : %s\n",
		    (ivmob->mmi & 0x4) ? "forbidden" : "authorized");
		printf("Charger         : %s\n", 
		    (ivmob->mmi & 0x02) ? "connected" : "disconnected");
		printf("Control selector: %s\n",
		    (ivmob->mmi & 0x01) ? "remote" : "local");
		printf("wheel speeds : %d %d %d %d\n",
		    ivmob->front_left_wheel_speed,
		    ivmob->front_right_wheel_speed,
		    ivmob->rear_left_wheel_speed,
		    ivmob->rear_right_wheel_speed);
		printf("wheel currents : %d %d %d %d\n",
		    ivmob->front_left_current,
		    ivmob->front_right_current,
		    ivmob->rear_left_current,
		    ivmob->rear_right_current);
		printf("\n");
		if (ivmob->operational_mode != IVMOB_MODE_ACTIVE)
			break;
		usleep(100000);
	}
	printf("**** switching to inactive \n");
	ivmobCmdInactive(ivmob, 2000);
	while (1) {
		printf("Temperature     : %d C\n"
		    "Battery         : %d/8\n"
		    "Mode            : %s\n"
		    "Safety code     : %s\n",
		    ivmob->temperature, ivmob->battery_level, 
		    ivmob_operating_mode(ivmob->operational_mode), 
		    ivmob_safety_code(ivmob->safety_error_code));
		printf("Motion          : %s\n",
		    (ivmob->mmi & 0x4) ? "forbidden" : "authorized");
		printf("Charger         : %s\n", 
		    (ivmob->mmi & 0x02) ? "connected" : "disconnected");
		printf("Control selector: %s\n",
		    (ivmob->mmi & 0x01) ? "remote" : "local");
		printf("\n");
		if (ivmob->operational_mode != IVMOB_MODE_ACTIVE)
			break;
		usleep(200000);
	}
	ivmobEnd(ivmob);
	return 0;
}
